// AnalysisModuleGraph.cpp: implementation of the CAnalysisModuleManager class.
//
//////////////////////////////////////////////////////////////////////
// #include "stdafx.h" // Unless using MFC, comment this line.

#include "AnalysisModuleManager.h"

/*
#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif
*/
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CAnalysisModuleManager::CAnalysisModuleManager()
{
	m_pOwnerInterface	= NULL;
	m_pszModuleDir		= NULL;
}

CAnalysisModuleManager::~CAnalysisModuleManager()
{
	UnloadModules();

	if ( m_pszModuleDir )
	{
		delete [] m_pszModuleDir;
		m_pszModuleDir = NULL;
	}
}

bool CAnalysisModuleManager::Initialize ( CAnalysisNotifyInterface* pOwnerInterface , char* pszModuleDir )
{
	if ( NULL == pszModuleDir ) return false;
	if ( NULL == pOwnerInterface ) return false;

	m_pOwnerInterface = pOwnerInterface;
	SetModuleDir ( pszModuleDir );

	m_ModuleGraph.Initialize( pOwnerInterface );
	return true;
}

void CAnalysisModuleManager::SetModuleDir ( char* pszModuleDir )
{
	int nLength = strlen ( pszModuleDir );
	if ( pszModuleDir )
	{
		delete [] m_pszModuleDir;
	}
	m_pszModuleDir = new char [ nLength + 1 ];
	memset ( m_pszModuleDir , 0x0 , nLength + 1 );
	memcpy ( m_pszModuleDir , pszModuleDir , nLength );
}

bool CAnalysisModuleManager::LoadModules()
{
	if ( NULL == m_pszModuleDir ) return false;
	CModuleInfo*	pInfo = NULL;
#ifdef	_LINUX_
	DIR*		pdirInfo		= NULL;
	struct dirent*	pdirEntry		= NULL;
	int		nLength			= 0;

	pdirInfo = opendir ( m_pszModuleDir );
	if ( pdirInfo )
	{
		while ( pdirEntry = readdir ( pdirInfo ) )
		{
			nLength	= strlen ( pdirEntry->d_name );
			if ( (nLength > 3) && memcmp ( pdirEntry->d_name + (nLength - 3) , ".so" , 3 ) == 0 )
			{
				pInfo = new CModuleInfo;
				pInfo->SetModuleFileName ( pdirEntry->d_name );
				if ( pInfo->DLOpen( m_pszModuleDir ) )
				{
					if ( pInfo->GetModuleInfo() )
					{
						m_listModule.push_back ( pInfo );
						// Print Module Info
						printf( "-- DLL Name = %s --\n" , pdirEntry->d_name );
						printf( "Module Id = %d , Rv = %d , Name = %s\n" , pInfo->m_uiId , pInfo->m_uiRevisionNo , pInfo->m_szName );
						printf( "Input Id = %d , Name = %s\n" , pInfo->m_uiInputId , pInfo->m_szInputName );
						printf( "Output Id = %d , Name = %s\n\n" , pInfo->m_uiOutputId , pInfo->m_szOutputName );
						//
						pInfo = NULL;
					}
					else
					{
						printf ( "ERROR : GetModuleInfo() in %s \n" , pdirEntry->d_name );
					}
				}
				else
				{
					printf ( "ERROR : Failed to DLOpen '%s module' \n" , pdirEntry->d_name );
				}

				if ( pInfo )
				{
					delete pInfo;
					pInfo = NULL;
				}
			}
		}
		closedir ( pdirInfo );
	}
#else 

#ifdef _GTK_
	char sz[1024];
	string str;
	for ( int i = 0 ; i < 10 ; i++ )
	{
		pInfo = new CModuleInfo;
		memset ( sz , 0x0 , sizeof ( sz ) );
		sprintf ( sz , "Module_%d.so" , i );
		pInfo->SetModuleFileName ( sz );
		pInfo->GetModuleInfo();
		
		m_listModule.push_back ( pInfo );
	}
#else
	CString str;
	for ( int i = 0 ; i < 10 ; i++ )
	{
		pInfo = new CModuleInfo;
		str.Format ( "Module_%d.so" , i );
		pInfo->SetModuleFileName ( (char *)str.operator const char*() );
		pInfo->GetModuleInfo();
		
		m_listModule.push_back ( pInfo );
	}
#endif //_GTK_


#endif
	return true;
}

void CAnalysisModuleManager::UnloadModules()
{
	m_ModuleGraph.Clear();

	CModuleInfo* pInfo = NULL;
	MODULE_ITER	itEnd = m_listModule.end();

	for ( MODULE_ITER it = m_listModule.begin() ; it != itEnd ; ++it )
	{
		pInfo = (*it);
		pInfo->DLClose();
		delete pInfo;
	}
	m_listModule.clear();
}

void CAnalysisModuleManager::InsertModuleIntoGraph ( CModuleInfo* pInfo )
{
	m_ModuleGraph.InsertModule ( pInfo );
}

void CAnalysisModuleManager::StartAnalysisProcess ( CSourceFilePin* pSourceFilePin )
{
	m_ModuleGraph.StartAnalysisProcess ( pSourceFilePin );
}

void CAnalysisModuleManager::StopAnalysisProcess()
{
	m_ModuleGraph.StopAnalysisProcess();
}
