#ifndef __MODULE_GRAPH_H__
#define __MODULE_GRAPH_H__

#include "include/AnalysisModuleInterface.h"
#include <list>
#include <math.h>

#ifdef _LINUX_
#include <assert.h>
#include <dlfcn.h>
#endif

using namespace std;

#define		PIN_TRACKER_MARGIN		8

/*
#ifndef ASSERT
#define ASSERT assert
#endif

#ifndef FORCEINLINE
#ifdef	__VISUALC__
#define	FORCEINLINE __forceinline
#else
#define FORCEINLINE inline
#endif
#endif
*/

// const char* dlError;

class CModuleInfo : public CAnalysisModuleInfo
{
public: 
	CModuleInfo() : CAnalysisModuleInfo()
	{
		m_phModule = NULL;
	}
	
	~CModuleInfo()
	{
		DLClose();
	}

	void SetModuleFileName ( char* pszFileName )
	{
		memset ( m_szModuleFileName , 0x0 , sizeof ( m_szModuleFileName ) );
		strncpy ( m_szModuleFileName , pszFileName , sizeof ( m_szModuleFileName ) - 1 );
	}

	bool DLOpen( char* pPath )
	{
		if ( m_phModule ) DLClose();

		const char* dlError = NULL;
		char* pAbsolutePath = NULL;

		int nLength = strlen( pPath ) + strlen( m_szModuleFileName  ) + 1;
		
		pAbsolutePath = new char[ nLength ];
		memset( pAbsolutePath , 0 , nLength );

		strncpy( pAbsolutePath , pPath , strlen( pPath ) );
		strncpy( pAbsolutePath + strlen( pPath ) , m_szModuleFileName , strlen( m_szModuleFileName ) );

#ifdef _LINUX_
		// printf( "Printf DLL path %s\n" , pAbsolutePath );
		m_phModule = dlopen ( pAbsolutePath , RTLD_LAZY );

		// m_phModule = dlopen ( m_szModuleFileName , RTLD_LAZY );
		dlError = dlerror();
		//print
#else

#endif
	
		if( pAbsolutePath )
		{
			delete [] pAbsolutePath;
			pAbsolutePath = NULL;
		}

		return (NULL == dlError);
	}

	bool DLClose()
	{
		int nReturnCode = 0;
		if ( m_phModule )
		{
#ifdef _LINUX_
			nReturnCode = dlclose ( m_phModule );
#else

#endif
			m_phModule = NULL;
		}
		return  (0 == nReturnCode);
	}

	bool GetModuleInfo()
	{
#ifdef	_LINUX_
		if ( !m_phModule ) return false;
		bool bRet = false;
		
		FP_GetModuleInfo pProc = ( FP_GetModuleInfo ) dlsym ( m_phModule , FN_GetModuleInfo );
		const char* dlError = dlerror();
		if ( pProc )
		{
			bRet = pProc ( this );
		}
#else 
		bool bRet = false;

		static int i = 0;
		static int n = 0;
		static int nInOutId = 0;
		m_uiId = n++;
		strncpy ( m_szName , m_szModuleFileName , sizeof ( m_szName ) - 1 );
		m_uiRevisionNo = 0;
		i++;

		m_uiInputId = nInOutId;
		if ( i % 3 )
		{
			nInOutId++;
			m_uiOutputId = nInOutId;
		}
		else
		{
			m_uiOutputId = nInOutId + 1;
		}
//		m_szInputName[256];
//		m_szInputDescription[1024];

//		m_szOutputName[256];
//		m_szOutputDescription[1024];
		
#endif
		return bRet;
	}	

	void Initialize ( CAnalysisNotifyInterface* pInterface , CModuleRelayInterface* pRelayInterface )
	{
#ifdef	_LINUX_
		if ( !m_phModule ) return;
		
		FP_Initialize pProc = ( FP_Initialize ) dlsym ( m_phModule , FN_Initialize );
		const char* dlError = dlerror();
		if ( pProc )
		{
			pProc ( pInterface , pRelayInterface );
		}
#endif
	}	

	bool StartProcess ( CInOutPinBase* pInputPin )
	{
		bool bRet = false;
#ifdef	_LINUX_
		if ( !m_phModule ) return false;
		
		FP_StartProcess pProc = ( FP_StartProcess ) dlsym ( m_phModule , FN_StartProcess );
		const char* dlError = dlerror();
		if ( pProc )
		{
			bRet = pProc ( pInputPin );
		}
#else 
		
#endif
		return bRet;
	}	

	bool StopProcess()
	{
		bool bRet = false;
#ifdef	_LINUX_
		if ( !m_phModule ) return false;
		
		FP_StopProcess pProc = ( FP_StopProcess ) dlsym ( m_phModule , FN_StopProcess );
		const char* dlError = dlerror();
		if ( pProc )
		{
			bRet = pProc();
		}
#else 
		
#endif
		return bRet;
	}	

	void SetOption ( char* pszOption )
	{
#ifdef	_LINUX_
		if ( !m_phModule ) return;
		
		FP_SetOption pProc = ( FP_SetOption ) dlsym ( m_phModule , FN_SetOption );
		const char* dlError = dlerror();
		if ( pProc )
		{
			pProc ( pszOption );
		}
#endif
	}	

	char* GetOption()
	{
		char* pszOption = NULL;
#ifdef	_LINUX_
		// if ( !m_phModule ) return ;
		if ( !m_phModule ) return NULL;
		
		FP_GetOption pProc = ( FP_GetOption ) dlsym ( m_phModule , FN_GetOption );
		const char* dlError = dlerror();
		if ( pProc )
		{
			pszOption = pProc();
		}
#endif
		return pszOption;
	}	


public:
	void*	m_phModule;
	char	m_szModuleFileName[256];
};

typedef		list<CModuleInfo *>		MODULE_LIST;
typedef		MODULE_LIST::iterator	MODULE_ITER;

class CModuleNode
{
public:
	CModuleNode()
	{
		m_pParentNode		= NULL;
		m_ppChildNodes		= NULL;
		m_pInfo				= NULL;
		m_nChildNodeCount	= 0;

		m_nX				= 0;
		m_nY				= 0;
		m_nWidth			= 200;
		m_nHeight			= 80;
	}
	
	~CModuleNode()
	{
		Clear();
	}

	void Clear()
	{
		if ( m_ppChildNodes )
		{
			delete [] m_ppChildNodes;
			m_ppChildNodes = NULL;
		}
		m_nChildNodeCount = 0;
	}

	bool IsExistNode ( CModuleNode* pNewChildNode )
	{
		for ( int i = 0 ; i < m_nChildNodeCount ; i++ )
		{
			if ( m_ppChildNodes[i]->m_pInfo->m_uiId == pNewChildNode->m_pInfo->m_uiId )
			{
				return true;
			}
		}
		return false;
	}

	bool AddChildNode ( CModuleNode* pNewChildNode )
	{
		if ( IsExistNode ( pNewChildNode ) )
		{
			return false;
		}

		if ( 0 == m_nChildNodeCount )
		{
			m_ppChildNodes = new CModuleNode* [ 1 ];
			m_ppChildNodes[0] = pNewChildNode;
		}
		else
		{
			CModuleNode** ppTempChildNodes = new CModuleNode* [ m_nChildNodeCount + 1 ];
			for ( int i = 0 ; i < m_nChildNodeCount ; i++ )
			{
				ppTempChildNodes[i] = m_ppChildNodes[i];
			}
			ppTempChildNodes[m_nChildNodeCount] = pNewChildNode;
			pNewChildNode->m_pParentNode = this;

			delete [] m_ppChildNodes;

			m_ppChildNodes = ppTempChildNodes;
		}
		m_nChildNodeCount ++;
		/*
		if ( m_nChildNodeCount == 0 )
		{
			ASSERT ( m_ppChildNodes == NULL );
		}
		*/
		return true;
	}

	CModuleNode* FindInChildNodes ( unsigned int uiId )
	{
		for ( int i = 0 ; i < m_nChildNodeCount ; i++ )
		{
			if ( m_ppChildNodes[i]->m_pInfo->m_uiId == uiId )
			{
				return m_ppChildNodes[i];
			}
		}
		return NULL;
	}


	void RemoveChildNode ( CModuleNode* pNode )
	{	
		if ( false == IsExistNode ( pNode ) ) return;

		if ( m_nChildNodeCount == 1 )
		{
			m_nChildNodeCount = 0;
			delete [] m_ppChildNodes;
			m_ppChildNodes = NULL;
		}
		else
		{
			int nIndex = 0;
			CModuleNode** ppTempChildNodes = new CModuleNode* [ m_nChildNodeCount - 1 ];
			for ( int i = 0 ; i < m_nChildNodeCount ; i++ )
			{
				if ( pNode != m_ppChildNodes[i] )
				{
					ppTempChildNodes[nIndex] = m_ppChildNodes[i];
					nIndex++;
				}
			}
			m_nChildNodeCount--;

			delete [] m_ppChildNodes;
			m_ppChildNodes = ppTempChildNodes;
		}
		/*
		if ( m_nChildNodeCount == 0 )
		{
			ASSERT ( m_ppChildNodes == NULL );
		}
		*/
	}

	bool PtInRect ( int& nX , int& nY )
	{
		if ( m_nX <= nX && nX <= m_nWidth + m_nX )
		{
			if ( m_nY <= nY && nY <= m_nHeight + m_nY )
			{
				return true;
			}
		}
		return false;
	}

	void Move ( int nX , int nY )
	{
		m_nX = nX;
		m_nY = nY;
	}

	void Offset ( int nX , int nY )
	{
		m_nX += nX;
		m_nY += nY;
	}

	void SetSize ( int nCX , int nCY )
	{
		m_nWidth = nCX;
		m_nHeight = nCY;
	}

	int GetRight()
	{
		return m_nX + m_nWidth;
	}

	int GetBottom()
	{
		return m_nY + m_nHeight;
	}

	void GetInputPoint ( int& nX , int& nY )
	{
		nX = m_nX;
		nY = m_nY + (m_nHeight / 2);
	}

	void GetOutputPoint ( int& nX , int& nY )
	{
		nX = m_nX + m_nWidth;
		nY = m_nY + (m_nHeight / 2);
	}

public:
	CModuleInfo*	m_pInfo;
	CModuleNode*	m_pParentNode;
	CModuleNode**	m_ppChildNodes;
	int				m_nChildNodeCount;

	int				m_nX;
	int				m_nY;
	int				m_nWidth;
	int				m_nHeight;
};

typedef		list<CModuleNode *>					MODULENODE_LIST;
typedef		MODULENODE_LIST::iterator			MODULENODE_ITER;
typedef		MODULENODE_LIST::reverse_iterator	MODULENODE_REVERSE_ITER;



class CModuleTrackingInterface
{
public:
	CModuleTrackingInterface()
	{
	}
	~CModuleTrackingInterface()
	{
	}

	virtual void OnTrackingModule ( int nDepth , CModuleNode* pParentNode , CModuleNode* pChildNode ) = 0;
};

class CModuleGraph : public CModuleRelayInterface
{
public:
	CModuleGraph();
	~CModuleGraph();

	void			Initialize ( CAnalysisNotifyInterface* pInterface );

	void			StartAnalysisProcess ( CSourceFilePin* pSourceFilePin );
	void			StopAnalysisProcess();

	CModuleNode*	FindNodeInList ( unsigned int uiId );

	CModuleNode*	HitTest ( int nX , int nY );

	CModuleNode*	HitTestInBFS ( CModuleNode* pNode , int& nX , int& nY );
	CModuleNode*	HitTestInDFS ( CModuleNode* pNode , int& nX , int& nY );
	CModuleNode*	HitTestInList ( int& nX , int& nY );

	CModuleNode*	PinHitTest ( int nX , int nY , bool& bPinIn );
	bool			ConnectionHitTest ( int nX , int nY , CModuleNode*& pParentNode ,  CModuleNode*& pChildNode );

	bool			IsExistNode ( CModuleInfo* pInfo );

	void			RemoveNodeInGraph ( CModuleNode* pNode );
	void			RemoveNodeInList ( CModuleNode* pNode );

	void			InsertModule ( CModuleInfo* pInfo );

	bool			ConnectModule ( unsigned int uiParentModuleId , unsigned int uiChildModuleId );
	bool			ConnectModule ( CModuleNode* pParentNode , CModuleNode* pChildNode );
	
	void			DisconnectModule ( unsigned int uiParentModuleId , unsigned int uiChildModuleId );
	void			DisconnectModule ( CModuleNode* pParentNode , CModuleNode* pChildNode );
	
	bool			PtInPointWithMargin ( int& nX , int& nY , int& nInX , int& nInY );

	void			Clear();

	virtual	void	OnRelayResult ( unsigned int uiModuleId , CInOutPinBase* pOutPin );


	inline void Begin()
	{
		m_iterModuleNode = m_listModuleNode.begin();
	}

	inline bool IsEnd()
	{
		return m_iterModuleNode == m_listModuleNode.end();
	}

	inline void NextIterator()
	{
		m_iterModuleNode++;
	}

	inline CModuleNode* GetIterator()
	{
		return (*m_iterModuleNode);
	}

	inline bool PtinLine ( double fAx , double fAy , double fBx , double fBy , double fPx , double fPy , double fEpsilon )
	{
		if ( abs ( fBx - fAx ) > abs ( fBy - fAy ) )
		{
			if ( fPx < min ( fAx , fBx ) || fPx > max ( fAx , fBx ) )
			{
				return false;
			}
		}
		else
		{
			if ( fPy < min ( fAy , fBy ) || fPy > max ( fAy , fBy ) )
			{
				return false;
			}
		}

		double fAbs_CA_x_CB	= abs ( (fAx - fPx) * (fBy - fPy) - (fBx - fPx) * ( fAy - fPy ) );
		double fAbs_AB		= sqrt ( (double)(abs((fBx - fAx) * (fBx - fAx) + (fBy - fAy) * (fBy - fAy) ) ) );
		double fDistance = fAbs_CA_x_CB / fAbs_AB;
		if ( fDistance < fEpsilon )
		{
			return true;
		}
		return false;
	}

protected:

public:
	MODULENODE_LIST	m_listModuleNode;
	MODULENODE_ITER	m_iterModuleNode;

	CAnalysisNotifyInterface* m_pNotifyInterface;
};

#endif // __MODULE_GRAPH_H__
