#include "ResultView.h"
#include <iostream>	// TEST CODE
using namespace std;
#include "MainFrame.h"
CResultView::CResultView():
VBox()
{
	pack_start( m_ComboBoxText, false, false );
	
	m_ComboBoxText.signal_changed().connect( sigc::mem_fun( *this, &CResultView::OnChangedComboBoxText ) );

	pack_start( m_ScrolledWindow );
	m_ScrolledWindow.set_policy( Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC );
	m_ScrolledWindow.add( m_TreeView );
	m_TreeView.set_grid_lines( Gtk::TREE_VIEW_GRID_LINES_BOTH );

	m_pTreeModelTotal = Gtk::ListStore::create( m_ColumnTotal );

	Reset();
	show_all_children();
}
CResultView::~CResultView()
{

}
void CResultView::OnChangedComboBoxText()
{
	ustring strActiveText = m_ComboBoxText.get_active_text();
	if ( strActiveText == "Total" )
	{
		SetTreeModelTotal();
	}
	else
	{
		SetTreeModelModule( strActiveText );
	}
}
void CResultView::SetTreeModelTotal()
{
	m_TreeView.remove_all_columns();
	m_TreeView.set_model( m_pTreeModelTotal );
	m_TreeView.append_column( "Source File Name", m_ColumnTotal.m_strSourceFileName );
	m_TreeView.append_column( "Code Line", m_ColumnTotal.m_nCodeLine );
	m_TreeView.append_column( "Module Count", m_ColumnTotal.m_nModuleCount );
}
void CResultView::SetTreeModelModule( ustring strModuleName )
{
	std::map < ustring, RefPtr< ListStore > >::iterator it = m_mapTreeModelModule.find( strModuleName );
	if ( it != m_mapTreeModelModule.end() )
	{
		m_TreeView.remove_all_columns();
		m_TreeView.set_model( it->second );
		m_TreeView.append_column( "Source File Name", m_ColumnTotal.m_strSourceFileName );
		m_TreeView.append_column( "Code Line", m_ColumnTotal.m_nCodeLine );
		m_TreeView.append_column( "Error Text", m_ColumnTotal.m_strErrorText );
	}
}
void CResultView::AddTreeModelModule( ustring strModuleName )
{
	m_ComboBoxText.append_text( strModuleName );
	m_mapTreeModelModule.insert( pair< ustring, RefPtr< ListStore > >( strModuleName, ListStore::create( m_ColumnModule ) ) );
}
void CResultView::Reset()
{
	m_ComboBoxText.clear_items();
	m_pTreeModelTotal->clear();
	if ( m_mapTreeModelModule.size() > 0 )
	{
		std::map < ustring, RefPtr< ListStore > >::iterator it;
		std::map < ustring, RefPtr< ListStore > >::iterator itEnd = m_mapTreeModelModule.end();
		for ( it = m_mapTreeModelModule.begin() ; it != itEnd ; ++it )
		{
			it->second->clear();
		}
		m_mapTreeModelModule.clear();
	}

	m_ComboBoxText.append_text( "Total" );
	m_ComboBoxText.set_active_text( "Total" );
}
string GetPureFileName(const string& str)    
{    
    string::size_type n = str.find_last_of("\\/");    
    return n == string::npos ? str : str.substr( n + 1, str.size() - n);    
}    
void CResultView::OnNotifyAnalysisResult ( unsigned int uiModuleId , CAnalysisResultPin* pAnalysisResultPin )
{
	char* pszModuleName = g_pmfMainFrame->GetModuleName( uiModuleId );
	if ( pszModuleName == NULL )
	{
		return;
	}
	ustring strModuleName	=	pszModuleName;
	
	std::map < ustring, RefPtr< ListStore > >::iterator it = m_mapTreeModelModule.find( strModuleName );
	if ( it == m_mapTreeModelModule.end() )
	{
		return;
	}
	m_strFileName	=	GetPureFileName( string( pAnalysisResultPin->m_pszFileName ) );
	m_nLineNo		=	pAnalysisResultPin->m_nLineNo;
	m_strErrorText	=	ustring( pAnalysisResultPin->m_pszErrorText );
	TreeModel::Row AddedRow = *( it->second->append() );
	AddedRow[ m_ColumnModule.m_strSourceFileName ]	= m_strFileName;
	AddedRow[ m_ColumnModule.m_nCodeLine ]			= m_nLineNo;
	AddedRow[ m_ColumnModule.m_strErrorText ]		= m_strErrorText;

	// Ż 
	m_bFindSameErrorInTotal = false;
	m_pTreeModelTotal->foreach_iter( sigc::mem_fun( *this, &CResultView::FindSameErrorInTotal ) );
	if ( m_bFindSameErrorInTotal == true )
	{
		m_rowFindSameErrorInTotal[ m_ColumnTotal.m_nModuleCount ] = m_rowFindSameErrorInTotal[ m_ColumnTotal.m_nModuleCount ] + 1;
	}
	else
	{
		AddedRow = *( m_pTreeModelTotal->append() );
		AddedRow[ m_ColumnTotal.m_strSourceFileName ] = m_strFileName;
		AddedRow[ m_ColumnTotal.m_nCodeLine ] = m_nLineNo;
		AddedRow[ m_ColumnTotal.m_nModuleCount ] = 1;
	}
}
bool CResultView::FindSameErrorInTotal( const TreeModel::iterator& itRow )
{
	TreeModel::Row TempRow = *itRow;
	if ( TempRow[ m_ColumnTotal.m_strSourceFileName ] == m_strFileName &&
		 TempRow[ m_ColumnTotal.m_nCodeLine ] == m_nLineNo  )
	{
		m_bFindSameErrorInTotal = true;
		m_rowFindSameErrorInTotal = TempRow;
		return true;
	}
	return false;
}


