#include <iostream>		// TEST CODE

#include "Main.h"
#include "MainFrame.h"
#include "SourceFileTree.h"
#include "ProjFileTreeFolder.h"
#include "ProjFileTreeFile.h"



using namespace std;	// TEST CODE

#define		NEW_FOLDER_NAME		"New Folder "
#define		NEW_FOLDER_MAX		500
#define		NEW_FOLDER_DEPTH	50


CSourceFileTree::CSourceFileTree() : TreeView()
{
	Glib::init();
	Gio::init();
	//Glib::thread_init();
	m_ptsTreeModel		= TreeStore::create( m_tmcColumns );
	m_ptsTreeSelection	= get_selection();

	set_model( m_ptsTreeModel );
	set_reorderable();

	this->signal_row_expanded		  ().connect( sigc::mem_fun( *this, &CSourceFileTree::on_row_expanded_ ) );
	this->signal_row_collapsed		  ().connect( sigc::mem_fun( *this, &CSourceFileTree::on_row_collapsed_ ) );

	Gtk::TreeView::Column* pColumn = Gtk::manage( new Gtk::TreeView::Column( "" ) );
	pColumn->pack_start( m_tmcColumns.m_colIcon, false );
	pColumn->pack_start( m_CellRendererText  );
	append_column( *pColumn );
	pColumn->set_cell_data_func( m_CellRendererText, sigc::mem_fun( *this, &CSourceFileTree::cell_data_func ) );
	m_CellRendererText.signal_editing_started().connect( sigc::mem_fun( *this, &CSourceFileTree::OnCellEditingStarted ) );
	m_CellRendererText.signal_edited().connect( sigc::mem_fun( *this, &CSourceFileTree::OnCellEdited ) );

	set_rubber_banding();
	set_enable_tree_lines();

	set_headers_visible( false );
	m_ptsTreeSelection->set_mode( SELECTION_MULTIPLE );

	show_all_children();
}
void CSourceFileTree::cell_data_func( Gtk::CellRenderer* renderer, const Gtk::TreeModel::iterator& iter )
{
	if(iter)
	{
		Gtk::TreeModel::Row row = *iter;
		m_CellRendererText.property_text() = row[ m_tmcColumns.m_colName ];
	}
}
CSourceFileTree::~CSourceFileTree()
{

}
void CSourceFileTree::SetProjectName( const ustring& strProjectName )
{
	TreeNodeChildren::iterator itRoot = m_ptsTreeModel->children().begin();
	if ( itRoot )
	{
		itRoot->set_value( m_tmcColumns.m_colName, strProjectName );
	}
}
void CSourceFileTree::Reset()
{
	m_ptsTreeModel->clear();
	TreeModel::Row tmRow = *( m_ptsTreeModel->append() );
	tmRow[ m_tmcColumns.m_colItemType ] = SFT_ITEM_TYPE_PROJECT;
	tmRow[ m_tmcColumns.m_colName	  ] = g_pmfMainFrame->GetProjectName();
	tmRow[ m_tmcColumns.m_colIcon	  ] = render_icon( Gtk::StockID( Gtk::Stock::SELECT_ALL ), Gtk::ICON_SIZE_MENU);
}
void CSourceFileTree::Save()
{

}
void CSourceFileTree::SaveAs( RefPtr< FileOutputStream > pFileOutputStream )
{
	m_pFileOutputStream = pFileOutputStream;
	m_nChildCount = 0;
	m_nChildCount = int( m_ptsTreeModel->children().begin()->children().size() );
	pFileOutputStream->write( &m_nChildCount, sizeof( m_nChildCount ) );
	cout << "m_nChildCount: " << m_nChildCount << endl;
	if ( m_nChildCount > 0 )
	{
		m_pSelectedFile.clear();
		m_ptsTreeModel->foreach_iter( sigc::mem_fun( *this, &CSourceFileTree::SaveRow ) );
		int nFileNameLength;
		if ( m_pSelectedFile )
		{
			string strFileName = m_pSelectedFile->get_path();
			nFileNameLength = int( strFileName.size() );
			pFileOutputStream->write( &nFileNameLength, sizeof( nFileNameLength ) );
			pFileOutputStream->write( strFileName );
		}
		else
		{
			nFileNameLength = 0;
			pFileOutputStream->write( &nFileNameLength, sizeof( nFileNameLength ) );
		}
	}
}
bool CSourceFileTree::SaveRow( const TreeModel::iterator& itRow )
{
	int nItemType = itRow->get_value( m_tmcColumns.m_colItemType );
	int nChildCount = int( itRow->children().size() );
	ustring strItemName = itRow->get_value( m_tmcColumns.m_colName );
	RefPtr< File > pFile = itRow->get_value( m_tmcColumns.m_colFile );
	ustring strFileName;
	bool bOpen;
	switch( nItemType )
	{
	case SFT_ITEM_TYPE_PROJECT: break;
	case SFT_ITEM_TYPE_FOLDER:
		{
			CProjFileTreeFolder ProjFileTreeFolder;
			ProjFileTreeFolder.Save( m_pFileOutputStream, nChildCount, strItemName );
		}
		break;
	case SFT_ITEM_TYPE_FILE:
		{
			strFileName = pFile->get_path();
			bOpen = g_pmfMainFrame->IsOpened( pFile );
			if ( g_pmfMainFrame->IsSelected( pFile ) == true )
			{
				m_pSelectedFile = pFile;
			}
			CProjFileTreeFile ProjFileTreeFile;
			ProjFileTreeFile.Save( m_pFileOutputStream, nChildCount, strItemName, strFileName, bOpen );
		}
		break;
	}
	return false;
}
void CSourceFileTree::Open( RefPtr< FileInputStream > pFileInputStream )
{
	m_pFileInputStream = pFileInputStream;
	int nChildCount;
	m_pFileInputStream->read( &nChildCount, sizeof( nChildCount ) );
	Open( m_ptsTreeModel->children().begin()->children(), nChildCount );
	int nFileNameLength;
	m_pFileInputStream->read( &nFileNameLength, sizeof( nFileNameLength ) );
	if ( nChildCount > 0 && nFileNameLength > 0 )
	{
		char* pszFileName = new char [ nFileNameLength + 1 ];
		m_pFileInputStream->read( pszFileName, nFileNameLength );
		pszFileName [ nFileNameLength ] = 0;
		RefPtr< File > pFile = File::create_for_path( pszFileName );
		delete [] pszFileName;
		g_pmfMainFrame->SelectPage( pFile );
		cout << pFile->get_path() << endl;
	}
}
void CSourceFileTree::Open( TreeNodeChildren tnParent, int nItemCount )
{
	TreeModel::iterator tmIter;
	int nItemIndex;
	int nItemType;
	int nChildCount;
	CProjFileTreeItem* pProjFileTreeItem;
	for ( nItemIndex = 0 ; nItemIndex < nItemCount ; ++nItemIndex )
	{
		tmIter = m_ptsTreeModel->append( tnParent );
		m_pFileInputStream->read( &nItemType, sizeof( nItemType ) );
		switch( nItemType )
		{
		case PROJ_FILE_TREE_FOLDER:
			{
				tmIter->set_value( m_tmcColumns.m_colItemType, int( SFT_ITEM_TYPE_FOLDER ) );
				tmIter->set_value( m_tmcColumns.m_colIcon, render_icon( Gtk::StockID( Gtk::Stock::DIRECTORY ), Gtk::ICON_SIZE_MENU) );
				pProjFileTreeItem = new CProjFileTreeFolder;
				pProjFileTreeItem->Load( m_pFileInputStream );
			}
			break;
		case PROJ_FILE_TREE_FILE:
			{
				tmIter->set_value( m_tmcColumns.m_colItemType, int( SFT_ITEM_TYPE_FILE ) );
				tmIter->set_value( m_tmcColumns.m_colIcon, render_icon( Gtk::StockID( Gtk::Stock::FILE ), Gtk::ICON_SIZE_MENU) );
				CProjFileTreeFile* pProjFileTreeFile = new CProjFileTreeFile;
				pProjFileTreeItem = pProjFileTreeFile;
				pProjFileTreeItem->Load( m_pFileInputStream );
				RefPtr< File > pFile = File::create_for_path( pProjFileTreeFile->GetFileName() );
				tmIter->set_value( m_tmcColumns.m_colFile, pFile );
				if ( pProjFileTreeFile->GetOpen() == true )
				{
					g_pmfMainFrame->GetViewManager()->AddSourceTextView( pFile, pProjFileTreeItem->GetItemName(), false );
				}
			}
			break;
		}
		tmIter->set_value( m_tmcColumns.m_colName, ustring( pProjFileTreeItem->GetItemName() ) );
		nChildCount = pProjFileTreeItem->GetChildCount();
		delete pProjFileTreeItem;
		Open( tmIter->children(), nChildCount );
	}
}
bool CSourceFileTree::on_button_release_event( GdkEventButton* pebEventButton )
{
	if ( pebEventButton->button == 3 )
	{
		vector<TreeModel::Path> vecPath = m_ptsTreeSelection->get_selected_rows();
		vector<TreeModel::Path>::const_iterator itPath;
		vector<TreeModel::Path>::const_iterator itPathEnd = vecPath.end();
		TreeModel::Row tmRow;
		unsigned int auiItemType[ SFT_ITEM_TYPE_COUNT ]  = { 0 };
		for ( itPath = vecPath.begin() ; itPath != itPathEnd ; ++itPath )
		{
			tmRow	=	*( m_ptsTreeModel->get_iter( *itPath ) );
			++auiItemType[ tmRow[ m_tmcColumns.m_colItemType ] ];
		}

		Menu::MenuList& mlMenuList = m_mwPopupMenu.items();
		mlMenuList.clear();
		if ( auiItemType[ SFT_ITEM_TYPE_PROJECT ] )
		{
			if ( auiItemType[ SFT_ITEM_TYPE_FOLDER ] == 0 && auiItemType[ SFT_ITEM_TYPE_FILE ] == 0 )
			{
				mlMenuList.push_back( Menu_Helpers::MenuElem( "Add Folder", sigc::mem_fun( *this, &CSourceFileTree::OnMenuAddFolder ) ) );
				mlMenuList.push_back( Menu_Helpers::MenuElem( "Add File", sigc::mem_fun( *this, &CSourceFileTree::OnMenuAddFile ) ) );
				mlMenuList.push_back( Menu_Helpers::MenuElem( "Rename", sigc::mem_fun( *this, &CSourceFileTree::OnMenuRename ) ) );
			}
		}
		else
		{
			if ( auiItemType[ SFT_ITEM_TYPE_FOLDER ] > 0 && auiItemType[ SFT_ITEM_TYPE_FILE ] == 0 )
			{
				if ( auiItemType[ SFT_ITEM_TYPE_FOLDER ] == 1 )
				{
					mlMenuList.push_back( Menu_Helpers::MenuElem( "Add Folder", sigc::mem_fun( *this, &CSourceFileTree::OnMenuAddFolder ) ) );
					mlMenuList.push_back( Menu_Helpers::MenuElem( "Add File", sigc::mem_fun( *this, &CSourceFileTree::OnMenuAddFile ) ) );
					mlMenuList.push_back( Menu_Helpers::MenuElem( "Rename", sigc::mem_fun( *this, &CSourceFileTree::OnMenuRename ) ) );
				}
				mlMenuList.push_back( Menu_Helpers::MenuElem( "Remove", sigc::mem_fun( *this, &CSourceFileTree::OnMenuRemove ) ) );
			}
			else if ( auiItemType[ SFT_ITEM_TYPE_FOLDER ] == 0 && auiItemType[ SFT_ITEM_TYPE_FILE ] > 0 )
			{
				mlMenuList.push_back( Menu_Helpers::MenuElem( "Open", sigc::mem_fun( *this, &CSourceFileTree::OnMenuOpen ) ) );
				mlMenuList.push_back( Menu_Helpers::MenuElem( "Remove", sigc::mem_fun( *this, &CSourceFileTree::OnMenuRemove ) ) );
				mlMenuList.push_back( Menu_Helpers::MenuElem( "Analysis", sigc::mem_fun( *this, &CSourceFileTree::OnMenuAnalysis ) ) );
			}
			else if ( auiItemType[ SFT_ITEM_TYPE_FOLDER ] > 0 && auiItemType[ SFT_ITEM_TYPE_FILE ] > 0 )
			{
				mlMenuList.push_back( Menu_Helpers::MenuElem( "Remove", sigc::mem_fun( *this, &CSourceFileTree::OnMenuRemove ) ) );
			}
		}
		mlMenuList.push_back( Menu_Helpers::MenuElem( "Stop Analysis", sigc::mem_fun( *this, &CSourceFileTree::OnMenuStopAnalysis ) ) );
		if ( mlMenuList.empty() == false )
		{
			m_mwPopupMenu.popup( pebEventButton->button, pebEventButton->time );
		}
	}
	return TreeView::on_button_release_event( pebEventButton );
}
void CSourceFileTree::on_row_activated( const TreeModel::Path& tmPath, TreeViewColumn* ptvColumn )
{
	TreeModel::Row tmRow = *( m_ptsTreeModel->get_iter( tmPath ) );
	if ( tmRow[ m_tmcColumns.m_colItemType ] == SFT_ITEM_TYPE_FILE )
	{
		g_pmfMainFrame->GetViewManager()->AddSourceTextView( tmRow[ m_tmcColumns.m_colFile ], tmRow[ m_tmcColumns.m_colName ] );
		g_pmfMainFrame->SetModified();
	}
	TreeView::on_row_activated( tmPath, ptvColumn );
}
void CSourceFileTree::OnMenuAddFolder()
{
	TreeModel::Path tmPath							=	*( m_ptsTreeSelection->get_selected_rows().begin() );
	if ( tmPath.get_depth() >= NEW_FOLDER_DEPTH )
	{
		MessageDialog msgDialog( "Can't create a new sub folder.\nPlease do it at lower depth." );
		msgDialog.set_title( "Error" );
		msgDialog.set_transient_for( *g_pmfMainFrame );
		msgDialog.run();
		return;
	}
	TreeNodeChildren tncNode						=	m_ptsTreeModel->get_iter( tmPath )->children();
	TreeNodeChildren::iterator	itNode				=	tncNode.begin();
	TreeNodeChildren::iterator	itNodeEnd			=	tncNode.end();
	size_t						sizeNewFolderName	=	strlen( NEW_FOLDER_NAME );
	string						strNumber;
	stringstream				strsPureNumber;
	int							nFolderNumber;
	vector< int >				vecNewFolderNumber;
	for ( ; itNode != itNodeEnd ; ++itNode )
	{
		if ( itNode->get_value( m_tmcColumns.m_colItemType ) == SFT_ITEM_TYPE_FOLDER )
		{
			const ustring&	strName = itNode->get_value( m_tmcColumns.m_colName );
			if ( strName.size() > sizeNewFolderName )
			{
				strNumber = strName.substr( sizeNewFolderName );
				strsPureNumber.str( "" );
				strsPureNumber.clear();
				strsPureNumber << atoi( strNumber.c_str() );
				if ( strNumber == strsPureNumber.str() )
				{
					strsPureNumber >> nFolderNumber;
					if ( nFolderNumber >= 1 && nFolderNumber <= NEW_FOLDER_MAX )
					{
						vecNewFolderNumber.push_back( nFolderNumber );
					}
				}
			}
		}
	}
	sort( vecNewFolderNumber.begin(), vecNewFolderNumber.end() );
	int nNewFolderNumber;
	vector< int >::iterator itNumber = vecNewFolderNumber.begin();
	if ( vecNewFolderNumber.empty() == false && *itNumber == 1)
	{
		vector< int >::iterator itNumberEnd = --vecNewFolderNumber.end();
		for ( ; itNumber != itNumberEnd ; ++itNumber )
		{
			if ( *( itNumber + 1 ) - *itNumber > 1 )
			{
				break;
			}
		}
		nNewFolderNumber = *itNumber + 1;
	}
	else
	{
		nNewFolderNumber = 1;
	}
	if ( nNewFolderNumber > NEW_FOLDER_MAX )
	{
		MessageDialog msgDialog( "Can't create a new folder.\nPlease rename or remove the existing folders." );
		msgDialog.set_title( "Error" );
		msgDialog.set_transient_for( *g_pmfMainFrame );
		msgDialog.run();
		return;
	}
	stringstream strsNewFolderName;
	strsNewFolderName << NEW_FOLDER_NAME << nNewFolderNumber;
	TreeModel::Row tmRow		=	*( m_ptsTreeModel->prepend( tncNode ) );
	tmRow[ m_tmcColumns.m_colItemType ] = SFT_ITEM_TYPE_FOLDER;
	tmRow[ m_tmcColumns.m_colName	  ] = strsNewFolderName.str();
	tmRow[ m_tmcColumns.m_colIcon	  ] = render_icon( Gtk::StockID( Gtk::Stock::DIRECTORY ), Gtk::ICON_SIZE_MENU);
	tmRow[ m_tmcColumns.m_bExpand	  ] = false;
	expand_to_path( tmPath );
	set_cursor( TreePath( tmRow ) );
	OnMenuRename();
	g_pmfMainFrame->SetModified();
}

void CSourceFileTree::OnMenuAddFile()
{
	FileChooserDialog fcdDialog( "Please choose files" );
	fcdDialog.set_select_multiple();
	fcdDialog.set_transient_for( *g_pmfMainFrame );
	fcdDialog.add_button( Stock::OPEN, RESPONSE_OK );
	fcdDialog.add_button( Stock::CANCEL, RESPONSE_CANCEL );
	FileFilter ffFilter;
	ffFilter.set_name( "Text files" );
	ffFilter.add_mime_type( "text/plain" );
	fcdDialog.add_filter( ffFilter );
	if ( fcdDialog.run() == RESPONSE_OK )
	{
		SListHandle< ustring > listFile				=	fcdDialog.get_filenames();
		SListHandle< ustring >::iterator itFile		=	listFile.begin();
		SListHandle< ustring >::iterator itFileEnd	=	listFile.end();
		RefPtr< File > pfileFile;
		TreeModel::Path tmPath		=	*( m_ptsTreeSelection->get_selected_rows().begin() );
		TreeNodeChildren tncNode	=	m_ptsTreeModel->get_iter( tmPath )->children();
		TreeModel::Row tmRow;
		bool bFind;
		for ( ; itFile != itFileEnd ; ++itFile )
		{
			pfileFile							=	File::create_for_path( *itFile );
			m_ptsTreeModel->foreach_iter( sigc::bind( sigc::mem_fun( *this, &CSourceFileTree::FindFile ), pfileFile, &bFind ) );
			if ( bFind == true )
			{
				continue;
			}
			tmRow								=	*( m_ptsTreeModel->append( tncNode ) );
			tmRow[ m_tmcColumns.m_colItemType ] =	SFT_ITEM_TYPE_FILE;
			tmRow[ m_tmcColumns.m_colName	  ] =	pfileFile->get_basename();
			tmRow[ m_tmcColumns.m_colFile	  ] =	pfileFile;
			tmRow[ m_tmcColumns.m_colIcon	  ] = render_icon( Gtk::StockID( Gtk::Stock::FILE ), Gtk::ICON_SIZE_MENU);
			tmRow[ m_tmcColumns.m_bExpand	  ] = false;
			expand_row( tmPath,false );
		}
	}
	g_pmfMainFrame->SetModified();
}
bool CSourceFileTree::FindFile( const TreeModel::iterator& itTreeModel, RefPtr< File > pfileFile, bool* pbFind )
{
	if ( itTreeModel->get_value( m_tmcColumns.m_colItemType ) == SFT_ITEM_TYPE_FILE )
	{
		*pbFind = pfileFile->equal( itTreeModel->get_value( m_tmcColumns.m_colFile ) );
	}
	else
	{
		*pbFind = false;
	}
	return *pbFind;
}
void CSourceFileTree::OnMenuRemove()
{
	vector<TreeModel::Path> vecPath = m_ptsTreeSelection->get_selected_rows();
	vector<TreeModel::Path>::const_reverse_iterator itPath;
	vector<TreeModel::Path>::const_reverse_iterator itPathEnd = vecPath.rend();
	TreeModel::Row tmRow;
	for ( itPath = vecPath.rbegin() ; itPath != itPathEnd ; ++itPath )
	{
		m_ptsTreeModel->erase( m_ptsTreeModel->get_iter( *itPath ) );
	}
	g_pmfMainFrame->SetModified();
}
void CSourceFileTree::OnMenuOpen()
{
	vector<TreeModel::Path> vecPath = m_ptsTreeSelection->get_selected_rows();
	vector<TreeModel::Path>::const_iterator itPath;
	vector<TreeModel::Path>::const_iterator itPathEnd = vecPath.end();
	TreeModel::Row tmRow;
	for ( itPath = vecPath.begin() ; itPath != itPathEnd ; ++itPath )
	{
		tmRow		=	*( m_ptsTreeModel->get_iter( *itPath ) );
		if ( tmRow[ m_tmcColumns.m_colItemType ] == SFT_ITEM_TYPE_FILE )
		{
			g_pmfMainFrame->GetViewManager()->AddSourceTextView( tmRow[ m_tmcColumns.m_colFile ], tmRow[ m_tmcColumns.m_colName ] );
		}
	}
	g_pmfMainFrame->SetModified();
}
void CSourceFileTree::OnMenuRename()
{
	m_CellRendererText.property_editable() = true;
	set_cursor( *( m_ptsTreeSelection->get_selected_rows().begin() ), *get_column( 0 ), true );
	m_CellRendererText.property_editable() = false;
}
void CSourceFileTree::OnMenuAnalysis()
{
	cout << "Analysis" << endl;
	vector<TreeModel::Path> vecPath = m_ptsTreeSelection->get_selected_rows();
	vector<TreeModel::Path>::const_iterator itPath;
	vector<TreeModel::Path>::const_iterator itPathEnd = vecPath.end();
	TreeModel::Row tmRow;
	RefPtr< File > pFile;
	SourceFilePin.m_nFileCount = 0;
	string strFileNameList;
	for ( itPath = vecPath.begin() ; itPath != itPathEnd ; ++itPath )
	{
		tmRow	=	*( m_ptsTreeModel->get_iter( *itPath ) );
		pFile = tmRow[ m_tmcColumns.m_colFile ];
		strFileNameList += Glib::locale_from_utf8( pFile->get_parse_name() );
		strFileNameList += ",";
		++SourceFilePin.m_nFileCount;
	}
	SourceFilePin.m_pszFileName = new char [ strFileNameList.size() + 1 ];
	strncpy( SourceFilePin.m_pszFileName, strFileNameList.c_str(), strFileNameList.size() );
	SourceFilePin.m_pszFileName[ strFileNameList.size() - 1 ] = '\0';

	cout << SourceFilePin.m_pszFileName << endl;
	cout << SourceFilePin.m_nFileCount << endl;

	// Result, Error, Process   

	CModuleGraph* pModuleGraph = g_pmfMainFrame->GetAnalysisModuleManager()->GetModuleGraph();
	CModuleInfo* pModuleInfo;
	CResultView* pResultView = g_pmfMainFrame->GetResultManager()->GetResultView();
	g_pmfMainFrame->GetResultManager()->Reset();
	for ( pModuleGraph->Begin() ; pModuleGraph->IsEnd() == false ; pModuleGraph->NextIterator() )
	{
		pModuleInfo = pModuleGraph->GetIterator()->m_pInfo;
		if ( pModuleInfo->m_uiOutputId == IOP_ANALYSIS_RESULT )
		{
			pResultView->AddTreeModelModule( pModuleInfo->m_szName );
		}
	}
	sleep(0);

	g_pmfMainFrame->GetAnalysisModuleManager()->StartAnalysisProcess( &SourceFilePin );
	//return;

	m_bStopThread = false;
	//Glib::Thread::create( sigc::mem_fun( *this, &CSourceFileTree::StartAnalysisThread ), false );
	//Glib::Thread::self();
}
void CSourceFileTree::StartAnalysisThread()
{
	//  
	g_pmfMainFrame->GetAnalysisModuleManager()->StartAnalysisProcess( &SourceFilePin );
	/*
	int i;
	for(i = 0; i<50; ++i)
	{
		if ( m_bStopThread == true)
		{
			break;
		}
		cout << "Thread ... " << i << endl;
		Glib::usleep( 100000 );
		// Ʈ \
		// 
		// μ  +++
	}
	//*/
	g_pmfMainFrame->GetResultManager()->GetResultView()->SetTreeModelTotal();
}
void CSourceFileTree::OnMenuStopAnalysis()
{
	//m_bStopThread = true;
	g_pmfMainFrame->GetAnalysisModuleManager()->StopAnalysisProcess();
}
void CSourceFileTree::OnCellEditingStarted( CellEditable* pceCellEditable, const ustring& strPath )
{
	m_strOldLabel = m_ptsTreeModel->get_iter( strPath )->get_value( m_tmcColumns.m_colName );
}
void CSourceFileTree::OnCellEdited( const ustring& strPath, const ustring& strNew )
{
	MessageDialog msgDialog( "" );
	msgDialog.set_transient_for( *g_pmfMainFrame );
	msgDialog.set_title( "Error" );
	TreeModel::iterator itEditedRow				= m_ptsTreeModel->get_iter( strPath );

	if ( itEditedRow->get_value( m_tmcColumns.m_colName ) != strNew &&
		 itEditedRow->get_value( m_tmcColumns.m_colItemType ) == SFT_ITEM_TYPE_FILE )
	{
		msgDialog.set_message( "Can't rename label of files." );
		msgDialog.run();
		itEditedRow->set_value( m_tmcColumns.m_colName, m_strOldLabel );
		return;
	}
	const TreeNodeChildren& tncChildren			= itEditedRow->parent()->children();
	TreeNodeChildren::iterator itChildren		= tncChildren.begin();
	TreeNodeChildren::iterator itChildrenEnd	= tncChildren.end();
	ustring strRowName;
	for ( ; itChildren != itChildrenEnd ; ++itChildren )
	{
		if ( itChildren.equal( itEditedRow ) == true )
		{
			continue;
		}
		if ( strNew == itChildren->get_value( m_tmcColumns.m_colName ) )
		{
			msgDialog.set_message( "Can't rename the label.\nPlease do any other name." );
			msgDialog.run();
			itEditedRow->set_value( m_tmcColumns.m_colName, m_strOldLabel );
			return;
		}
	}
	string strNewConversion = strNew;
	string::size_type sizeOffset = 0;
	while( 1 )
	{   
		sizeOffset = strNewConversion.find( "\n", sizeOffset );
		if( string::npos == sizeOffset )
		{
			break;
		}
		else
		{
			strNewConversion.replace( sizeOffset, 1, "" );
		}
	}
	string::size_type sizeBegin	= strNewConversion.find_first_not_of( " \t" );
	string::size_type sizeEnd		= strNewConversion.find_last_not_of( " \t" );
	if ( sizeBegin == string::npos )
	{
		msgDialog.set_message( "Please enter a name.");
		msgDialog.run();
		itEditedRow->set_value( m_tmcColumns.m_colName, m_strOldLabel );
	}
	else
	{
		ustring strNewFinal = strNewConversion.substr( sizeBegin, sizeEnd - sizeBegin + 1 );
		if ( strNewFinal == m_strOldLabel )
		{
			return;
		}
		if ( itEditedRow->get_value( m_tmcColumns.m_colItemType ) == SFT_ITEM_TYPE_PROJECT )
		{
			g_pmfMainFrame->SetProjectName( strNewFinal );
		}
		else
		{
			itEditedRow->set_value( m_tmcColumns.m_colName, strNewFinal );
		}
		g_pmfMainFrame->SetModified();
	}
}

void CSourceFileTree::on_row_expanded_( const TreeIter& itItem, const TreePath& pathItem )
{
	TreeRow rowItem = *itItem;
	if ( rowItem[ m_tmcColumns.m_colItemType ] == SFT_ITEM_TYPE_FOLDER )
	{
		rowItem[ m_tmcColumns.m_colIcon  ] = render_icon( StockID( Stock::OPEN ), ICON_SIZE_MENU );
		rowItem[ m_tmcColumns.m_bExpand ] = true;
	}

	// ش    Expand ° true ٸ Ȯ

	const TreeNodeChildren& Children = itItem->children();
	TreeIter itItemChild;
	TreeIter itItemChildEnd = Children.end();

	for ( itItemChild = Children.begin() ; itItemChild != itItemChildEnd ; ++itItemChild )
	{
		rowItem = *itItemChild;
		if ( rowItem[ m_tmcColumns.m_bExpand ] == true )
		{
			// expand_row ȣ ش  signal_row_expanded ñ׳ ٽ ߻ǹǷ  ȿ 
			expand_row( TreePath( rowItem ), false );
		}
	}
}
void CSourceFileTree::on_row_collapsed_( const TreeIter& itItem, const TreePath& pathItem )
{
	TreeRow rowItem = *itItem;
	if ( rowItem[ m_tmcColumns.m_colItemType ] == SFT_ITEM_TYPE_FOLDER )
	{
		rowItem[ m_tmcColumns.m_colIcon  ] = render_icon( StockID( Stock::DIRECTORY ), ICON_SIZE_MENU );
		rowItem[ m_tmcColumns.m_bExpand ] = false;
	}
}

